"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateBinCommand = void 0;
var tslib_1 = require("tslib");
var path_1 = tslib_1.__importDefault(require("path"));
var ora_1 = tslib_1.__importDefault(require("ora"));
var fs_extra_1 = tslib_1.__importDefault(require("fs-extra"));
var utils_1 = require("../../shared/utils");
var setup_1 = require("./setup");
var package_const_1 = require("../../shared/constants/package.const");
var messages_1 = require("../../shared/messages");
var scaffolding_helpers_1 = require("./scaffolding.helpers");
var generate_helpers_1 = require("./generate.helpers");
var templateOptions = Object.keys(setup_1.generateSetup);
var generateBinCommand = function (prog) {
    prog
        .command('generate <pkg> [path]')
        .describe("Generate a new package.\n    Package name is required.\n    Path is optional. The path argument indicates where the folder with the package name should be generated.")
        .alias('g')
        .example('generate packageName')
        .example('generate packageName .')
        .example('generate packageName workspaces')
        .option('t, template', "Specify a template.\n     Available templates: [" + templateOptions.join(', ') + "]\n     \n     ")
        .example("generate packageName --template " + templateOptions[0])
        .action(function (pkgName, pathArg, _a) {
        var template = _a.template;
        return tslib_1.__awaiter(void 0, void 0, void 0, function () {
            var packageName, packageTemplateType, rootDir, packageJsonPath, packageJson, bootSpinner, workspacesInDeclaration, packageSetupPath, packageDir, author, templateConfig, packageJsonPreset, err_1, dependencies, preparingSpinner, err_2;
            return tslib_1.__generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        packageName = pkgName;
                        packageTemplateType = template;
                        return [4 /*yield*/, utils_1.findHostDirectory()];
                    case 1:
                        rootDir = _b.sent();
                        packageJsonPath = path_1.default.resolve(rootDir, package_const_1.PACKAGE_JSON);
                        return [4 /*yield*/, fs_extra_1.default.readJSON(packageJsonPath)];
                    case 2:
                        packageJson = (_b.sent());
                        bootSpinner = ora_1.default(messages_1.generateMessage.generating(pkgName));
                        _b.label = 3;
                    case 3:
                        _b.trys.push([3, 10, , 11]);
                        workspacesInDeclaration = utils_1.getPackagesFromDeclaration(packageJson.workspaces);
                        packageSetupPath = pathArg || utils_1.getNextPackageSetupPath(workspacesInDeclaration);
                        return [4 /*yield*/, scaffolding_helpers_1.preventFolderCollisions({
                                basePath: path_1.default.resolve(rootDir, packageSetupPath),
                                name: pkgName,
                                onPromptMessage: function (name) { return messages_1.generateMessage.failed(name); },
                                onPromptInitial: function (name) { return messages_1.generateMessage.copy(name); }
                            })];
                    case 4:
                        // TODO: new algorithm of package name? get all names and prevent duplication
                        packageName = _b.sent();
                        packageDir = path_1.default.resolve(rootDir, packageSetupPath, packageName);
                        return [4 /*yield*/, generate_helpers_1.getPackageTemplateType(template, function () {
                                bootSpinner.fail(messages_1.generateMessage.invalidTemplate(template));
                            })];
                    case 5:
                        packageTemplateType = _b.sent();
                        return [4 /*yield*/, scaffolding_helpers_1.getAuthor()];
                    case 6:
                        author = _b.sent();
                        bootSpinner.start();
                        utils_1.setNpmAuthorName(author);
                        return [4 /*yield*/, scaffolding_helpers_1.copyTemplate({
                                dir: packageDir,
                                bin: 'generate',
                                template: packageTemplateType
                            })];
                    case 7:
                        _b.sent();
                        process.chdir(packageDir);
                        templateConfig = setup_1.generateSetup[packageTemplateType];
                        packageJsonPreset = generate_helpers_1.composePackageJson({
                            author: author,
                            name: packageName,
                            license: packageJson.license,
                            hostName: packageJson.name,
                            template: templateConfig
                        });
                        return [4 /*yield*/, scaffolding_helpers_1.createPackageJson({
                                dir: packageDir,
                                preset: packageJsonPreset
                            })];
                    case 8:
                        _b.sent();
                        return [4 /*yield*/, generate_helpers_1.updatePackageJsonWorkspacesDeclaration({
                                packageJsonPath: packageJsonPath,
                                packageJson: packageJson,
                                packageName: packageName,
                                packages: workspacesInDeclaration,
                                setupPath: packageSetupPath
                            })];
                    case 9:
                        _b.sent();
                        bootSpinner.succeed(messages_1.generateMessage.successful(packageName));
                        return [3 /*break*/, 11];
                    case 10:
                        err_1 = _b.sent();
                        bootSpinner.fail(messages_1.generateMessage.failed(packageName));
                        utils_1.logError(err_1);
                        process.exit(1);
                        return [3 /*break*/, 11];
                    case 11:
                        dependencies = setup_1.generateSetup[packageTemplateType].dependencies;
                        preparingSpinner = ora_1.default(messages_1.generateMessage.preparingPackage(packageName, dependencies.sort())).start();
                        _b.label = 12;
                    case 12:
                        _b.trys.push([12, 15, , 16]);
                        return [4 /*yield*/, scaffolding_helpers_1.sortPackageJson()];
                    case 13:
                        _b.sent();
                        // TODO: doesn't log anything on error
                        return [4 /*yield*/, generate_helpers_1.buildPackage()];
                    case 14:
                        // TODO: doesn't log anything on error
                        _b.sent();
                        preparingSpinner.succeed(messages_1.generateMessage.successfulConfigure());
                        console.log(messages_1.generateMessage.preparedPackage(packageName));
                        return [3 /*break*/, 16];
                    case 15:
                        err_2 = _b.sent();
                        preparingSpinner.fail(messages_1.generateMessage.failedConfigure());
                        utils_1.logError(err_2);
                        process.exit(1);
                        return [3 /*break*/, 16];
                    case 16: return [2 /*return*/];
                }
            });
        });
    });
};
exports.generateBinCommand = generateBinCommand;
